package com.jiuqi.script.rhino;

import java.lang.reflect.Method;
import java.lang.reflect.Member;
import org.mozilla.javascript.FunctionObject;
import java.lang.reflect.Modifier;
import org.mozilla.javascript.BaseFunction;
import java.util.Date;
import org.mozilla.javascript.NativeJavaObject;
import com.jiuqi.script.IScriptObject;
import java.util.Calendar;
import org.mozilla.javascript.IdFunctionObject;
import org.mozilla.javascript.IdScriptableObject;
import org.mozilla.javascript.Undefined;
import org.mozilla.javascript.UniqueTag;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.RhinoException;
import com.jiuqi.script.CompileException;
import org.mozilla.javascript.JavaScriptException;
import com.jiuqi.script.EvalException;
import org.mozilla.javascript.ScriptableObject;
import com.jiuqi.script.ScriptFactory;
import com.jiuqi.script.ScriptManager;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.Context;
import com.jiuqi.script.Script;

public class RhinoScript implements Script
{
    private Context cx;
    private Scriptable scope;
    
    static {
        ScriptManager.registerScriptFactory((ScriptFactory)new RhinoScriptFactory());
    }
    
    public RhinoScript() {
        this.cx = Context.enter();
        this.scope = (Scriptable)this.cx.initStandardObjects((ScriptableObject)null, false);
        this.regDefaultFunction();
        Context.exit();
    }
    
    public Object evaluate(final String script, final String sourceName) throws CompileException, EvalException {
        this.cx = Context.enter();
        try {
            final Object result = this.cx.evaluateString(this.scope, script, sourceName, 1, (Object)null);
            return this.convertObject(result);
        }
        catch (final JavaScriptException e) {
            throw new EvalException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), (Throwable)e);
        }
        catch (final RhinoException e2) {
            throw new CompileException(e2.getMessage(), e2.sourceName(), e2.lineNumber(), e2.columnNumber(), (Throwable)e2);
        }
        finally {
            Context.exit();
        }
    }
    
    public Object call(final String name, final Object[] args) throws NoSuchMethodException, CompileException, EvalException {
        Object[] param = args;
        if (param == null) {
            param = new Object[0];
        }
        this.convertArgument(param);
        this.cx = Context.enter();
        try {
            final Function fun = this.getFunctionFromScope(this.scope, name);
            if (fun != null) {
                final Object result = fun.call(this.cx, this.scope, (Scriptable)fun, param);
                return this.convertObject(result);
            }
            throw new NoSuchMethodException("没有该函数:" + name);
        }
        catch (final JavaScriptException e) {
            throw new EvalException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), (Throwable)e);
        }
        catch (final RhinoException e2) {
            throw new CompileException(e2.getMessage(), e2.sourceName(), e2.lineNumber(), e2.columnNumber(), (Throwable)e2);
        }
        finally {
            Context.exit();
        }
    }
    
    public void compile(final String script, final String sourceName) throws CompileException {
        this.cx = Context.enter();
        try {
            this.cx.evaluateString(this.scope, script, sourceName, 1, (Object)null);
        }
        catch (final RhinoException e) {
            throw new CompileException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), (Throwable)e);
        }
        finally {
            Context.exit();
        }
        Context.exit();
    }
    
    public Object get(final String name) {
        final Object o = this.getValueFromScope(this.scope, name);
        if (o instanceof UniqueTag || o instanceof Undefined) {
            return null;
        }
        return o;
    }
    
    public void put(final String name, final Object value) {
        this.putValueToScope(this.scope, name, value);
    }
    
    public Object toJava(final Object obj) {
        if (obj == null) {
            return null;
        }
        if (obj instanceof RhinoScriptObject) {
            return ((RhinoScriptObject)obj).getScriptObject();
        }
        if (!(obj instanceof IdScriptableObject)) {
            return obj;
        }
        final IdScriptableObject result = (IdScriptableObject)obj;
        if (result.getClassName().equals("Date")) {
            final IdFunctionObject v = (IdFunctionObject)ScriptableObject.getProperty((Scriptable)result, "getTime");
            final Object m = result.execIdCall(v, Context.getCurrentContext(), this.scope, (Scriptable)result, (Object[])null);
            final long millis = ((Double)m).longValue();
            final Calendar c = Calendar.getInstance();
            c.setTimeInMillis(millis);
            return c;
        }
        return result;
    }
    
    public Object toScript(final Object obj) {
        if (obj == null) {
            return null;
        }
        if (obj instanceof IScriptObject) {
            return new RhinoScriptObject((IScriptObject)obj);
        }
        if (obj instanceof IdScriptableObject) {
            return obj;
        }
        if (obj instanceof NativeJavaObject) {
            final Object result = ((NativeJavaObject)obj).unwrap();
            if (result instanceof IScriptObject) {
                return new RhinoScriptObject((IScriptObject)result);
            }
            if (result instanceof Calendar) {
                final Calendar c = (Calendar)result;
                return Context.getCurrentContext().newObject(this.scope, "Date", new Object[] { new Long(c.getTimeInMillis()) });
            }
            if (result instanceof Date) {
                final Date d = (Date)result;
                return Context.getCurrentContext().newObject(this.scope, "Date", new Object[] { new Long(d.getTime()) });
            }
        }
        return obj;
    }
    
    private Function getFunctionFromScope(final Scriptable scope, String name) {
        String parentName = name;
        Scriptable parent = scope;
        if (name == null) {
            return null;
        }
        while (name.indexOf(".") != -1) {
            parentName = parentName.substring(0, name.indexOf("."));
            final Object o = parent.get(parentName, scope);
            if (!(o instanceof Scriptable)) {
                return null;
            }
            parent = (Scriptable)o;
            name = name.substring(name.indexOf(".") + 1);
        }
        final Object o = parent.get(name, scope);
        if (o instanceof Function) {
            return (Function)o;
        }
        return null;
    }
    
    private Object getValueFromScope(final Scriptable scope, String name) {
        String parentName = name;
        Scriptable parent = scope;
        if (name == null) {
            return null;
        }
        while (name.indexOf(".") != -1) {
            parentName = parentName.substring(0, name.indexOf("."));
            final Object o = parent.get(parentName, scope);
            if (!(o instanceof Scriptable)) {
                return null;
            }
            parent = (Scriptable)o;
            name = name.substring(name.indexOf(".") + 1);
        }
        return parent.get(name, scope);
    }
    
    private void putValueToScope(final Scriptable scope, String name, final Object value) {
        String parentName = name;
        Scriptable parent = scope;
        if (name == null) {
            return;
        }
        while (name.indexOf(".") != -1) {
            parentName = parentName.substring(0, name.indexOf("."));
            final Object o = parent.get(parentName, scope);
            if (!(o instanceof Scriptable)) {
                return;
            }
            parent = (Scriptable)o;
            name = name.substring(name.indexOf(".") + 1);
        }
        if (value instanceof IScriptObject) {
            parent.put(name, scope, (Object)new RhinoScriptObject((IScriptObject)value));
        }
        else {
            parent.put(name, scope, value);
        }
    }
    
    private void convertArgument(final Object[] args) {
        if (args == null) {
            return;
        }
        for (int i = 0; i < args.length; ++i) {
            if (args[i] instanceof Calendar) {
                final Calendar c = (Calendar)args[i];
                args[i] = this.cx.newObject(this.scope, "Date", new Object[] { new Long(c.getTimeInMillis()) });
            }
        }
    }
    
    private Object convertObject(final Object result) {
        if (result instanceof ScriptableObject) {
            final ScriptableObject o = (ScriptableObject)result;
            if (o.getClassName().equals("CustomScriptObject")) {
                return ((RhinoScriptObject)o).getScriptObject();
            }
            if (o.getClassName().equals("Date")) {
                final BaseFunction v = (BaseFunction)ScriptableObject.getProperty((Scriptable)o, "getTime");
                final Object m = v.call(this.cx, this.scope, (Scriptable)o, (Object[])null);
                final long millis = ((Double)m).longValue();
                final Calendar c = Calendar.getInstance();
                c.setTimeInMillis(millis);
                return c;
            }
            return o;
        }
        else {
            if (result instanceof NativeJavaObject) {
                final Object o2 = ((NativeJavaObject)result).unwrap();
                return o2;
            }
            return result;
        }
    }
    
    private void regDefaultFunction() {
        if (this.scope == null) {
            return;
        }
        final Class c = ScriptHelper.class;
        final ScriptHelper u = new ScriptHelper(this.scope);
        final Method[] methods = c.getDeclaredMethods();
        for (int i = 0; i < methods.length; ++i) {
            if (!methods[i].getName().equals("getClassName")) {
                if (Modifier.isPublic(methods[i].getModifiers())) {
                    this.scope.put(methods[i].getName(), this.scope, (Object)new FunctionObject(methods[i].getName(), (Member)methods[i], (Scriptable)u));
                }
            }
        }
    }
}
